# candyboot

Simple and lightweight UEFI stub loader for the Linux kernel.

Candyboot can be used to produce UEFI bundle executables which include a kernel,
cmdline and initrd. A UEFI system can run this executable directly to boot into
a regular Linux setup. The bundle can be signed with Secure Boot keys and
ensures that no portion of the load chain has been tampered.

## Current state

- Boots into a regular setup.
- No support for DeviceTree (e.g.: likely won't work on ARM yet).
- Needs an audit by a second pair of eyes.

The current implementation creates a copy of the initrd in memory when the
kernel requests it. It is likely that this can be improved in future by simply
passing a pointer to the existing in-memory copy.

## Mode of operation

Candyboot needs to be bundled into a single executable with `.linux`, `.initrd`
and `.cmdline` sections.

When executed, candyboot shall:

- Retain a pointer to the `.initrd` section.
- Register a UEFI boot service which exposes the `LoadFile2` UEFI protocol and
  with the `LINUX_INITRD_DEVICE_PATH` device path.
- Encode the cmdline into UTF-16 (this is what the kernel expects when executed
  via a UEFI stub).
- Execute the kernel image in the `.linux` section.

Once the kernel starts, it will detect the service exposed with the
`LINUX_INITRD_DEVICE_PATH` device path and use it to load an initrd. Candyboot's
registered service shall made a copy into a new buffer, and return a pointer to
it to the kernel.

Candyboot does not rely on the handover protocol, which was deprecated as of
Linux v5.10, in late 2020.

## Hints

- The `initrd=` kernel command line parameter shall be ignored by the kernel.

## Security model

Candyboot only operates on its own in-memory image. It is expected that the
previous loader in the chain (or the firmware) validated the signature of
candyboot itself.

Given that the whole image was signed, the embedded kernel, initrd and cmdline
are considered trusted.

Candyboot takes no inputs. Therefore, it does not trust any inputs.

## Compatibility

Candyboot currently works with ukify:

```sh
# the .uname section is unused, but required by ukify.
ukify build \
  --linux path/to/vmlinuz \
  --initrd path/to/initramfs \
  --cmdline @path/to/cmdline \
  --uname "Linux Edge via CandyBoot" \
  --stub path/to/candyboot.efi \
  --output path/to/ESP/efi/boot/bootx64.efi
```

Candyboot does not currently work with `efi-mkuki` due to some hard-coded sizes.
See: <https://github.com/jirutka/efi-mkuki/issues/4>

## Trivia

- Candyboot is smaller than both gummiboot-efistub and systemd-efistub.
- Gummiboot is unmaintained, and depends on Linux's deprecated handover
  protocol. It also won't compile with recent versions of its dependencies.

## Licence

This project is licensed under the ISC licence. See LICENCE.md for details.
