#!/bin/sh
#
# Helper script to run a VM and see how things are working.

set -e

# First run: rustup target add x86_64-unknown-uefi
cargo build --target x86_64-unknown-uefi
STUB=target/x86_64-unknown-uefi/debug/candyboot.efi

mkdir -p ESP/efi/boot/
KERNEL=/efi/vmlinuz-edge
CMDLINE=$(realpath cmdline)
INITRD=/efi/initramfs-edge
OUTPUT=ESP/efi/boot/bootx64.efi

# # Assumes alignment = 4096 (check with objdump --private-headers)
# # => SectionAlignment	00001000
#
# TODO: need to calculate locations of sections and set them with --change-section-vma
# # use objdump --section-headers
#
# objcopy \
#   --preserve-dates \
#   --enable-deterministic-archives \
#   --add-section ".kernel=$KERNEL" --set-section-flags .kernel=code,readonly --change-section-vma .kernel+0x140020314\
#   --add-section ".cmdline=$CMDLINE" --set-section-flags .cmdline=data,readonly --change-section-vma .cmdline+0x140020314\
#   --add-section ".initrd=$INITRD" --set-section-flags .initrd=data,readonly --change-section-vma .initrd+0x140020314\
#   target/x86_64-unknown-uefi/debug/candyboot.efi \
#   ESP/efi/boot/bootx64.efi

# FIXME: test only (works, but has not cmdline, initrd, kernel)
# install -D target/x86_64-unknown-uefi/debug/candyboot.efi ESP/efi/boot/bootx64.efi

# FIXME: uses hardcoded offsets instead of calculating.
#        ends up using more space than needed, and will fail for very large kernel/initrd
# efi-mkuki \
#   -c $CMDLINE \
#   -o ESP/efi/boot/bootx64.efi \
#   -S target/x86_64-unknown-uefi/debug/candyboot.efi \
#   $KERNEL \
#   $INITRD

# install -D $STUB $OUTPUT
# KERNEL=vmlinuz-virt

ukify build \
  --linux "$KERNEL" \
  --initrd "$INITRD" \
  --cmdline "@$CMDLINE" \
  --uname "Linux Edge via CandyBoot" \
  --stub $STUB \
  --output $OUTPUT

# Hint: the default memory size is too small. Merely including the initrd fails
# because the whole binary won't fit into memory.
qemu-system-x86_64 -enable-kvm \
    -drive if=pflash,format=raw,readonly=on,file=/usr/share/OVMF/OVMF_CODE.fd \
    -drive if=pflash,format=raw,readonly=on,file=/usr/share/OVMF/OVMF_VARS.fd \
    -m 512M \
    -nographic \
    -drive format=raw,file=fat:rw:ESP
